@@ -34,7 +34,7 @@
 static void usage(char *argv[])
 {
 	fprintf(stderr, "%s [certificate (PEM or URL)] [private key URL] "
-            "[module] [conf]\n", argv[0]);
+		"[module] [conf]\n", argv[0]);
 }
 
 static void display_openssl_errors(int l)
@@ -109,7 +109,7 @@ int main(int argc, char *argv[])
 		goto end;
 	}
 
-	if (!ENGINE_ctrl_cmd_string(engine, "VERBOSE", NULL, 0)) {
+	if (!ENGINE_ctrl_cmd_string(engine, "DEBUG_LEVEL", "7", 0)) {
 		display_openssl_errors(__LINE__);
 		exit(1);
 	}
@@ -187,3 +187,5 @@ end:
 
 	return ret;
 }
+
+/* vim: set noexpandtab: */
